==> trircd-release-5.1.7-r.tar.gz: trircd-release-5.1.7-r/modules/m_trace.c <==
/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_trace.c
 *
 *   Copyright (C) 2000-2003 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: m_trace.c,v 5.8 2003/02/06 16:37:21 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "s_conf.h"
#include "h.h"

static char *token = TOK1_TRACE;

static struct Message _msgtab[] = {
    {MSG_TRACE, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_permission, m_trace, m_trace, m_trace}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.8 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
}
#else
void m_trace_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}
#endif

/*
 * * m_trace 
 *        parv[0] = sender prefix 
 *        parv[1] = servername
 */
int m_trace(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    dlink_node *ptr;
    aClient *acptr = NULL;
    aClass *cltmp;
    char *tname;

    tname = (parc > 1) ? parv[1] : me.name;

    if (parc > 2)
	if (hunt_server(cptr, sptr, ":%s %s %s :%s", TOK1_TRACE, 2, parc, parv))
	    return 0;

    switch (hunt_server(cptr, sptr, ":%s %s :%s", TOK1_TRACE, 1, parc, parv)) {
	case HUNTED_ISME:
	    break;
	default:
	    return 0;
    }

    if (IsPerson(sptr))
	sendto_lev(SPY_LEV, "TRACE requested by %s (%s@%s) [%s]",
		   sptr->name, sptr->user->username, sptr->user->host, sptr->user->server);

    for (ptr = lclient_list.head; ptr; ptr = ptr->next) {
    char *name;
    int class;
	if (!(acptr = ptr->data))
	    continue;
	if (IsInvisible(acptr) && !(MyConnect(sptr) && IsAnOper(sptr)) && !IsAnOper(acptr) &&
	    (acptr != sptr))
	    continue;
	name = get_client_name(acptr, FALSE);
	class = get_client_class(acptr);
	switch (acptr->status) {
	    case STAT_CONNECTING:
		send_me_numeric(sptr, RPL_TRACECONNECTING, class, name);
		break;
	    case STAT_HANDSHAKE:
		send_me_numeric(sptr, RPL_TRACEHANDSHAKE, class, name);
		break;
	    case STAT_UNKNOWN:
		send_me_numeric(sptr, RPL_TRACEUNKNOWN, class, name,
				acptr->firsttime ? timeofday - acptr->firsttime : -1);
		break;
	    case STAT_CLIENT:
		if (IsAnOper(acptr))
		    send_me_numeric(sptr, RPL_TRACEOPERATOR, class, name,
				    timeofday - acptr->lasttime);
		break;
	    case STAT_ME:
	    case STAT_SERVER:
		send_me_numeric(sptr, RPL_TRACESERVER, class, acptr->fd,
				acptr->fd, name,
				*(acptr->serv->bynick) ? acptr->serv->bynick : "*");
		break;
	    case STAT_SERVICE:
		send_me_numeric(sptr, RPL_TRACESERVICE, class, name,
				acptr->service->sflags, timeofday - acptr->lasttime, acptr->info);
		break;
	    default:
		send_me_numeric(sptr, RPL_TRACENEWTYPE, name);
		break;
	}
    }

    if (MyConnect(sptr)) {
	for (ptr = unknown_list.head; ptr; ptr = ptr->next) {
    char *name;
    int class;
	    if (!(acptr = ptr->data))
		continue;
	    name = get_client_name(acptr, FALSE);
	    class = get_client_class(acptr);
	    switch (acptr->status) {
		case STAT_CONNECTING:
		    send_me_numeric(sptr, RPL_TRACECONNECTING, class, name);
		    break;
		case STAT_HANDSHAKE:
		    send_me_numeric(sptr, RPL_TRACEHANDSHAKE, class, name);
		    break;
		case STAT_UNKNOWN:
		    send_me_numeric(sptr, RPL_TRACEUNKNOWN, class, name,
				    acptr->firsttime ? timeofday - acptr->firsttime : -1);
		    break;
		case STAT_CLIENT:
		    if (IsAnOper(acptr))
			send_me_numeric(sptr, RPL_TRACEOPERATOR, class, name,
					timeofday - acptr->lasttime);
		    break;
		case STAT_ME:
		case STAT_SERVER:
		    send_me_numeric(sptr, RPL_TRACESERVER, class, acptr->fd,
				    acptr->fd, name,
				    *(acptr->serv->bynick) ? acptr->serv->bynick : "*");
		    break;
		case STAT_SERVICE:
		    send_me_numeric(sptr, RPL_TRACESERVICE, class, name,
				    acptr->service->sflags, timeofday - acptr->lasttime,
				    acptr->info);
		    break;
		default:
		    send_me_numeric(sptr, RPL_TRACENEWTYPE, name);
		    break;
	    }
	}
    }

    for (cltmp = classes; cltmp; cltmp = cltmp->next)
	if (cltmp->links > 0)
	    send_me_numeric(sptr, RPL_TRACECLASS, cltmp->class, cltmp->links);
    send_me_numeric(sptr, RPL_ENDOFTRACE, tname);
    return 0;
}

==> trircd-release-5.7.3-r.tar.gz: trircd-release-5.7.3-r/modules/m_trace.c <==
/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_trace.c
 *
 *   Copyright (C) 2000-2003 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: m_trace.c,v 1.4 2003/07/17 10:07:13 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "s_conf.h"
#include "h.h"

static char *token = TOK1_TRACE;

static struct Message _msgtab[] = {
    {MSG_TRACE, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_permission, m_trace, m_trace, m_trace}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 1.4 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
}
#else
void m_trace_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}
#endif

/*
 * * m_trace 
 *        parv[0] = sender prefix 
 *        parv[1] = servername
 */
int m_trace(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    dlink_node *ptr;
    aClient *acptr = NULL;
    aClass *cltmp;
    char *tname;

    tname = (parc > 1) ? parv[1] : me.name;

    if (parc > 2)
	if (hunt_server(cptr, sptr, ":%s %s %s :%s", TOK1_TRACE, 2, parc, parv))
	    return 0;

    switch (hunt_server(cptr, sptr, ":%s %s :%s", TOK1_TRACE, 1, parc, parv)) {
	case HUNTED_ISME:
	    break;
	default:
	    return 0;
    }

    if (IsPerson(sptr))
	sendto_lev(SPY_LEV, "TRACE requested by %^C [%s]", sptr, sptr->user->server);

    for (ptr = lclient_list.head; ptr; ptr = ptr->next) {
    char *name;
    int class;
	if (!(acptr = ptr->data))
	    continue;
	if (IsInvisible(acptr) && !(MyConnect(sptr) && IsSeeHidden(sptr)) && !IsAnOper(acptr) &&
	    (acptr != sptr))
	    continue;
	name = get_client_name(acptr, FALSE);
	class = get_client_class(acptr);
	switch (acptr->status) {
	    case STAT_CONNECTING:
		send_me_numeric(sptr, RPL_TRACECONNECTING, class, name);
		break;
	    case STAT_HANDSHAKE:
		send_me_numeric(sptr, RPL_TRACEHANDSHAKE, class, name);
		break;
	    case STAT_UNKNOWN:
		send_me_numeric(sptr, RPL_TRACEUNKNOWN, class, name,
				acptr->firsttime ? timeofday - acptr->firsttime : -1);
		break;
	    case STAT_CLIENT:
		if (IsAnOper(acptr))
		    send_me_numeric(sptr, RPL_TRACEOPERATOR, class, name,
				    timeofday - acptr->lasttime);
		break;
	    case STAT_ME:
	    case STAT_SERVER:
		send_me_numeric(sptr, RPL_TRACESERVER, class, acptr->fd,
				acptr->fd, name,
				*(acptr->serv->bynick) ? acptr->serv->bynick : "*");
		break;
	    case STAT_SERVICE:
		send_me_numeric(sptr, RPL_TRACESERVICE, class, name,
				acptr->service->sflags, timeofday - acptr->lasttime, acptr->info);
		break;
	    default:
		send_me_numeric(sptr, RPL_TRACENEWTYPE, name);
		break;
	}
    }

    if (MyConnect(sptr)) {
	for (ptr = unknown_list.head; ptr; ptr = ptr->next) {
    char *name;
    int class;
	    if (!(acptr = ptr->data))
		continue;
	    name = get_client_name(acptr, FALSE);
	    class = get_client_class(acptr);
	    switch (acptr->status) {
		case STAT_CONNECTING:
		    send_me_numeric(sptr, RPL_TRACECONNECTING, class, name);
		    break;
		case STAT_HANDSHAKE:
		    send_me_numeric(sptr, RPL_TRACEHANDSHAKE, class, name);
		    break;
		case STAT_UNKNOWN:
		    send_me_numeric(sptr, RPL_TRACEUNKNOWN, class, name,
				    acptr->firsttime ? timeofday - acptr->firsttime : -1);
		    break;
		case STAT_CLIENT:
		    if (IsAnOper(acptr))
			send_me_numeric(sptr, RPL_TRACEOPERATOR, class, name,
					timeofday - acptr->lasttime);
		    break;
		case STAT_ME:
		case STAT_SERVER:
		    send_me_numeric(sptr, RPL_TRACESERVER, class, acptr->fd,
				    acptr->fd, name,
				    *(acptr->serv->bynick) ? acptr->serv->bynick : "*");
		    break;
		case STAT_SERVICE:
		    send_me_numeric(sptr, RPL_TRACESERVICE, class, name,
				    acptr->service->sflags, timeofday - acptr->lasttime,
				    acptr->info);
		    break;
		default:
		    send_me_numeric(sptr, RPL_TRACENEWTYPE, name);
		    break;
	    }
	}
    }

    for (cltmp = ConnectionClasses; cltmp; cltmp = cltmp->next)
	if (cltmp->links > 0)
	    send_me_numeric(sptr, RPL_TRACECLASS, cltmp->number, cltmp->links);
    send_me_numeric(sptr, RPL_ENDOFTRACE, tname);
    return 0;
}
